@@ -1022,6 +1022,76 @@ g_string_insert_unichar (GString *string,
   return string;
 }
 
+/**
+ * g_string_overwrite:
+ * @string: a #GString
+ * @pos: the position at which to start overwriting
+ * @val: the string that will overwrite the @string starting at @pos
+ * 
+ * Overwrites part of a string, lengthening it if necessary.
+ * 
+ * Return value: @string
+ *
+ * Since: 2.14
+ **/
+GString *
+g_string_overwrite (GString     *string,
+		    gsize        pos,
+		    const gchar *val)
+{
+  g_return_val_if_fail (val != NULL, string);
+  return g_string_overwrite_len (string, pos, val, strlen (val));
+}
+
+/**
+ * g_string_overwrite_len:
+ * @string: a #GString
+ * @pos: the position at which to start overwriting
+ * @val: the string that will overwrite the @string starting at @pos
+ * @len: the number of bytes to write from @val
+ * 
+ * Overwrites part of a string, lengthening it if necessary. This function
+ * will work with embedded NULLs.
+ * 
+ * Return value: @string
+ *
+ * Since: 2.14
+ **/
+GString *
+g_string_overwrite_len (GString       *string,
+			gsize          pos,
+			const gchar   *val,
+			gssize         len)
+{
+  gsize end;
+
+  g_return_val_if_fail (string != NULL, NULL);
+
+  if (!len)
+    return string;
+
+  g_return_val_if_fail (val != NULL, string);
+  g_return_val_if_fail (pos <= string->len, string);
+
+  if (len < 0)
+    len = strlen (val);
+
+  end = pos + len;
+
+  if (end > string->len)
+    g_string_maybe_expand (string, end - string->len);
+
+  memcpy (string->str + pos, val, len);
+
+  if (end > string->len)
+    {
+      string->str[end] = '\0';
+      string->len = end;
+    }
+
+  return string;
+}
+
 /**
  * g_string_erase:
  * @string: a #GString
@@ -1190,17 +1260,58 @@ g_string_up (GString *string)
   return string;
 }
 
-static void
-g_string_append_printf_internal (GString     *string,
-				 const gchar *fmt,
-				 va_list      args)
+/**
+ * g_string_append_vprintf:
+ * @string: a #GString.
+ * @format: the string format. See the printf() documentation.
+ * @args: the list of arguments to insert in the output.
+ *
+ * Appends a formatted string onto the end of a #GString.
+ * This function is is similar to g_string_append_printf()
+ * except that the arguments to the format string are passed
+ * as a va_list.
+ *
+ * Since: 2.14
+ */
+void
+g_string_append_vprintf (GString     *string,
+			 const gchar *fmt,
+			 va_list      args)
 {
-  gchar *buffer;
-  gint length;
-  
-  length = g_vasprintf (&buffer, fmt, args);
-  g_string_append_len (string, buffer, length);
-  g_free (buffer);
+  va_list sneak;
+  gsize length;
+
+  g_return_if_fail (string != NULL);
+  g_return_if_fail (fmt != NULL);
+
+  G_VA_COPY (sneak, args);
+  length = g_printf_string_upper_bound (fmt, sneak);
+  va_end (sneak);
+
+  g_string_maybe_expand (string, length);
+  length = g_vsnprintf (string->str + string->len, length, fmt, args);
+  string->len += length;
+}
+
+/**
+ * g_string_vprintf:
+ * @string: a #GString.
+ * @format: the string format. See the printf() documentation.
+ * @Varargs: the parameters to insert into the format string.
+ *
+ * Writes a formatted string into a #GString.  This function
+ * is similar to g_string_printf() except that the arguments to
+ * the format string are passed as a va_list.
+ *
+ * Since: 2.14
+ */
+void
+g_string_vprintf (GString     *string,
+		  const gchar *fmt,
+		  va_list      args)
+{
+  g_string_truncate (string, 0);
+  g_string_append_vprintf (string, fmt, args);
 }
 
 /**
@@ -1240,7 +1351,7 @@ g_string_printf (GString *string,
   g_string_truncate (string, 0);
 
   va_start (args, fmt);
-  g_string_append_printf_internal (string, fmt, args);
+  g_string_append_vprintf (string, fmt, args);
   va_end (args);
 }
 
@@ -1276,7 +1387,7 @@ g_string_append_printf (GString *string,
   va_list args;
 
   va_start (args, fmt);
-  g_string_append_printf_internal (string, fmt, args);
+  g_string_append_vprintf (string, fmt, args);
   va_end (args);
 }
 
