@@ -80,6 +80,8 @@ static void    g_thread_cleanup (gpointer data);
 static void    g_thread_fail (void);
 static guint64 gettime (void);
 
+guint64        (*g_thread_gettime) (void) = gettime;
+
 /* Global variables */
 
 static GSystemThread zero_thread; /* This is initialized to all zero */
@@ -108,9 +110,8 @@ GThreadFunctions g_thread_functions_for_glib_use = {
   NULL,                                        /* thread_join */
   NULL,                                        /* thread_exit */
   NULL,                                        /* thread_set_priority */
-  NULL,                                         /* thread_self */
-  NULL,                                         /* thread_equal */
-  gettime                                      /* gettime */
+  NULL,                                        /* thread_self */
+  NULL                                         /* thread_equal */
 };
 
 /* Local data */
@@ -229,7 +230,10 @@ g_static_mutex_free (GStaticMutex* mutex)
   g_return_if_fail (mutex);
 
   /* The runtime_mutex is the first (or only) member of GStaticMutex,
-   * see both versions (of glibconfig.h) in configure.in */
+   * see both versions (of glibconfig.h) in configure.in. Note, that
+   * this variable is NULL, if g_thread_init() hasn't been called or
+   * if we're using the default thread implementation and it provides
+   * static mutexes. */
   runtime_mutex = ((GMutex**)mutex);
 
   if (*runtime_mutex)
@@ -554,8 +558,14 @@ gettime (void)
 #ifdef G_OS_WIN32
   guint64 v;
 
+  /* Returns 100s of nanoseconds since start of 1601 */
   GetSystemTimeAsFileTime ((FILETIME *)&v);
 
+  /* Offset to Unix epoch */
+  v -= G_GINT64_CONSTANT (116444736000000000);
+  /* Convert to nanoseconds */
+  v *= 100;
+
   return v;
 #else
   struct timeval tv;
